void setvoxel(int x, int y, int z);
void clrvoxel(int x, int y, int z);
unsigned char getvoxel(int x, int y, int z);
unsigned char inrange(int x, int y, int z);
void flpvoxel(int x, int y, int z);
void altervoxel(int x, int y, int z, int state);
void setplane_z(int z);
void clrplane_z(int z);
void setplane_x(int x);
void clrplane_x(int x);
void setplane_y(int y);
void clrplane_y(int y);
void setplane(unsigned char plane, int i, int state);
void setline_z(int x, int y, int z1, int z2);
void setline_x(int z, int y, int x1, int x2);
void setline_y(int z, int x, int y1, int y2);
void clrline_z(int x, int y, int z1, int z2);
void clrline_x(int z, int y, int x1, int x2);
void clrline_y(int z, int x, int y1, int y2);
void drawline_plane(int x2, int y2, int x1, int y1,unsigned char anchor);
void fill (unsigned char pattern);
void tmp2cube(void);
void cube2tmp(void);


// Setarea unui voxel
void setvoxel(int x, int y, int z)
{ 
	if (inrange(x, y, z))
		cube[z][y] |= (1 << x);
}

// Stergerea lui
void clrvoxel(int x, int y, int z)
{
	if (inrange(x, y, z))
	cube[z][y] &= ~(1 << x);
}

// Starea unui voxel
unsigned char getvoxel(int x, int y, int z)
{
	if (inrange(x, y, z))
	{
		if (cube[z][y] & (1 << x))
		{
			return 0x01;
		} else
		{
			return 0x00;
		}
	}
}


// Desenam in interiorul cubului
unsigned char inrange(int x, int y, int z)
{
	if (x >= 0 && x < 4 && y >= 0 && y < 4 && z >= 0 && z < 4)
	{
		return 0x01;
	} else
	{

		return 0x00;
	}
}

// alterarea starii unui voxel
void altervoxel(int x, int y, int z, int state)
{
	if (state == 1)
	{
		setvoxel(x,y,z);
	} else
	{
		clrvoxel(x,y,z);
	}
}

// Schimbarea starii unui voxel din 1 in 0 si invers
void flpvoxel(int x, int y, int z)
{
	if (inrange(x, y, z))
		cube[z][y] ^= (1 << x);
}


void setplane_z (int z)
{
	int i;
	for (i=0;i<4;i++)
		cube[z][i] = 0x0f;
}

void clrplane_z (int z)
{
	int iy;
	for (iy=0;iy<4;iy++)
		cube[z][iy] = 0x00;
}


void setplane_x (int x)
{
	int z;
	int y;
	for (z=0;z<4;z++)
	{
		for (y=0;y<4;y++)
		{
			cube[z][y] |= (0x01 << x);
		}
	}
}

void clrplane_x (int x)
{
	int z;
	int y;
	for (z=0;z<4;z++)
	{
		for (y=0;y<4;y++)
		{
			cube[z][y] &= ~(0x01 << x);
		}
	}
}

void setplane_y (int y)
{
	int iz;
	for (iz=0;iz<4;iz++)
		cube[iz][y] = 0x0f; 
}

// Sets all the leds on X and Z for a given Y
void clrplane_y (int y)
{
	int iz;
	for (iz=0;iz<4;iz++)
		cube[iz][y] = 0x00; 
}


void setplane(unsigned char plane, int i, int state)
{
	if (plane == "x")
	{
		if (state > 0)
		{
			setplane_x(i);
		} else
		{
			clrplane_x(i);
		}
	}
	
	if (plane == "y")
	{
		if (state > 0)
		{
			setplane_y(i);
		} else
		{
			clrplane_y(i);
		}
	}
	
	if (plane == "z")
	{
		if (state > 0)
		{
			setplane_z(i);
		} else
		{
			clrplane_z(i);
		}
	}
}



void line_z(int x, int y, int z1, int z2, int state)
{
	if (z1>z2)
	{
		int tmp;
		tmp = z1;
		z1= z2;
		z2 = tmp;
	}
	int i;
	for (i=z1;i<=z2;i++)
	{
		altervoxel(x,y,i,state);
	}
}

void line_x(int z, int y, int x1, int x2, int state)
{
	if (x1>x2)
	{
		int tmp;
		tmp = x1;
		x1= x2;
		x2 = tmp;
	}
	int i;
	for (i=x1;i<=x2;i++)
	{
		altervoxel(i,y,z,state);
	}
}

void line_y(int z, int x, int y1, int y2, int state)
{
	if (y1>y2)
	{
		int tmp;
		tmp = y1;
		y1= y2;
		y2 = tmp;
	}
	int i;
	for (i=y1;i<=y2;i++)
	{
			altervoxel(x,i,z,state);
	}
}


void drawline_plane(int x2, int y2, int x1, int y1,unsigned char anchor){
	int deltax = abs(x2 - x1);   
	int deltay = abs(y2 - y1);   
	int x = x1;                  
	int y = y1;                  
	int xinc1;
	int xinc2;
	int yinc1;
	int yinc2;
	int den;
	int num;
	int numadd;
	int numpixels;
	
	if (x2 >= x1)                
	{
		xinc1 = 1;
		xinc2 = 1;
	}
	else                          
	{
		xinc1 = -1;
		xinc2 = -1;
	}
	
	if (y2 >= y1)                 
	{
		yinc1 = 1;
		yinc2 = 1;
	}
	else                          
	{
		yinc1 = -1;
		yinc2 = -1;
	}
	
	if (deltax >= deltay)         
	{
		xinc1 = 0;  
		yinc2 = 0;  
		den = deltax;
		num = deltax / 2;
		numadd = deltay;
		numpixels = deltax; 
	}
	else                    
	{
		xinc2 = 0;          
		yinc1 = 0;          
		den = deltay;
		num = deltay / 2;
		numadd = deltax;
		numpixels = deltay; 
	}
	int curpixel;
	for (curpixel = 0; curpixel <= numpixels; curpixel++)
	{
		if (anchor == "z")
		{
			
			setvoxel(x, y, 3);
			setvoxel(x, y, 2);
			setvoxel(x, y, 1);
			setvoxel(x, y, 0);
		}
		
		if (anchor == "x")
		{
			
			setvoxel(x, 3, y);
			setvoxel(x, 2, y);
			setvoxel(x, 1, y);
			setvoxel(x, 0, y);
		}
		
		if (anchor == "y")
		{
			
			setvoxel(3, y, x);
			setvoxel(2, y, x);
			setvoxel(1, y, x);
			setvoxel(0, y, x);
		}
		
		num += numadd; 
		if (num >= den) 
		{
			num -= den;  
			x += xinc1;  
			y += yinc1;  
		}
		x += xinc2;      
		y += yinc2;      
	}
}


void tmp2cube(void)
{
	int y, z;
	for (z=0;z<4;z++)
	{
		for (y=0;y<4;y++)
		{
			cube[z][y] = tmpcube[z][y];
		}
	}
}


void cube2tmp(void)
{
	int y, z;
	for (z=0;z<4;z++)
	{
		for (y=0;y<4;y++)
		{
			tmpcube[z][y] = cube[z][y];
		}
	}
}


void fill (unsigned char pattern)
{
	int z;
	int y;
	for (z=0;z<4;z++)
	{
		for (y=0;y<4;y++)
		{
			cube[z][y] = pattern;
		}
	}
}



